#ifndef __JABYENGINE_TIMER_IO_HPP__
#define __JABYENGINE_TIMER_IO_HPP__
#include "ioport.hpp"

namespace JabyEngine {
    namespace Timer_IO {
        struct CounterMode   : public ComplexBitMap<uint32_t> {
            static constexpr auto SyncEnable       = Bit<uint32_t>(0);
            static constexpr auto FreeRun          = !SyncEnable;
            static constexpr auto SyncMode         = BitRange<uint32_t>::from_to(1, 2);
            static constexpr auto ResetAfterTarget = Bit<uint32_t>(3);
            static constexpr auto IRQAtTarget      = Bit<uint32_t>(4);
            static constexpr auto IRQAtMax         = Bit<uint32_t>(5);
            static constexpr auto IRQEveryTime     = Bit<uint32_t>(6);
            static constexpr auto IRQOneShot       = !IRQEveryTime;
            static constexpr auto IRQToggle        = Bit<uint32_t>(7);
            static constexpr auto IRQPulse         = !IRQToggle;            
            static constexpr auto ClockSource      = BitRange<uint32_t>::from_to(8, 9);
            static constexpr auto HasIRQRequest    = Bit<uint32_t>(10);
            static constexpr auto IsTargetReached  = Bit<uint32_t>(11);
            static constexpr auto IsMaxReached     = Bit<uint32_t>(12);
        };

        struct CounterTarget : public ComplexBitMap<uint32_t> {
            static constexpr auto CounterTargetValue = BitRange<uint32_t>::from_to(0, 15);
        };

        struct CounterValue : public ComplexBitMap<uint32_t> {
            static constexpr auto Value = BitRange<uint32_t>::from_to(0, 15);
        };

        struct __no_align Counter {
            VolatileBitMapPOD<CounterValue>  value;
            VolatileBitMapPOD<CounterMode>   mode;
            VolatileBitMapPOD<CounterTarget> target;
        private:
            uint32_t _unused;
        };

        static constexpr uintptr_t counter_base_adr(size_t ID) {
            return (0x1F801100 + (ID*0x10));
        }

        struct __no_align Counter0 : public Counter {
            struct SyncMode {
                static constexpr auto Pause_During_Hblank                 = CounterMode::SyncMode.with(0);
                static constexpr auto Zero_At_Hblank                      = CounterMode::SyncMode.with(1);
                static constexpr auto Zero_At_Hblank_Pause_Outside_Hblank = CounterMode::SyncMode.with(2);
                static constexpr auto Pause_Until_Hblank_Then_Freerun     = CounterMode::SyncMode.with(3);
            };

            struct Source {
                static constexpr auto System_Clock     = CounterMode::ClockSource.with(0);
                static constexpr auto Dot_Clock        = CounterMode::ClockSource.with(1);
                static constexpr auto System_Clock_Too = CounterMode::ClockSource.with(2);
                static constexpr auto Dot_Clock_Too    = CounterMode::ClockSource.with(3);
            };
        };

        struct __no_align Counter1 : public Counter {
            struct SyncMode {
                static constexpr auto Pause_During_Vblank                 = CounterMode::SyncMode.with(0);
                static constexpr auto Zero_At_Vblank                      = CounterMode::SyncMode.with(1);
                static constexpr auto Zero_At_Vblank_Pause_Outside_Vblank = CounterMode::SyncMode.with(2);
                static constexpr auto Pause_Until_Vblank_Then_Freerun     = CounterMode::SyncMode.with(3);
            };

            struct Source {
                static constexpr auto System_Clock     = CounterMode::ClockSource.with(0);
                static constexpr auto Hblank           = CounterMode::ClockSource.with(1);
                static constexpr auto System_Clock_Too = CounterMode::ClockSource.with(2);
                static constexpr auto Hblank_Too       = CounterMode::ClockSource.with(3);   
            };
        };

        struct __no_align Counter2 : public Counter {
            struct SyncMode {
                static constexpr auto Stop_Counter     = CounterMode::SyncMode.with(0);
                static constexpr auto Freerun          = CounterMode::SyncMode.with(1);
                static constexpr auto Freerun_Too      = CounterMode::SyncMode.with(2);
                static constexpr auto Stop_Counter_Too = CounterMode::SyncMode.with(3);
            };

            struct Source {
                static constexpr auto System_Clock           = CounterMode::ClockSource.with(0);
                static constexpr auto System_Clock_Too       = CounterMode::ClockSource.with(1);
                static constexpr auto System_Clock_Div_8     = CounterMode::ClockSource.with(2);
                static constexpr auto System_Clock_Div_8_Too = CounterMode::ClockSource.with(3);
            };
        };

        __declare_io_port_global_struct(struct Counter0, Counter0, counter_base_adr(0));
        __declare_io_port_global_struct(struct Counter1, Counter1, counter_base_adr(1));
        __declare_io_port_global_struct(struct Counter2, Counter2, counter_base_adr(2));
    }
}

#endif //!__JABYENGINE_TIMER_IO_HPP__