#include "../../include/GPU/gpu_internal.hpp"
#include <PSX/File/Processor/file_processor.hpp>
#include <PSX/Auxiliary/lz4_decompressor.hpp>
#include <PSX/GPU/gpu.hpp>
#include <stdio.h>

#ifdef JABYENGINE_PAL
    #include "splash_image_pal_boot.hpp"
#else
    #include "splash_image_ntsc_boot.hpp"
#endif //JABYENGINE_PAL

extern "C" uint32_t __boot_loader_end;

namespace JabyEngine {
    namespace boot {
        namespace GPU {
            using namespace JabyEngine::GPU;
            
            static size_t decompress_logo() {
                LZ4Decompressor lz4_decomp(reinterpret_cast<uint8_t*>(&__boot_loader_end));

                const auto [progress, bytes_ready] = lz4_decomp.process(ArrayRange(SplashScreen, sizeof(SplashScreen)), true);
                switch(progress) {
                    case Progress::InProgress:
                        printf("Decompressing still in progress... %llu\n", bytes_ready);
                        break;

                    case Progress::Error:
                        printf("Error decompressing!!!\n");
                        break;

                    case Progress::Done:
                        printf("Done decompressing: %llu Bytes ready\n", bytes_ready);
                        break;
                }

                return bytes_ready;
            }

            void display_logo() {
                const auto bytes_ready = decompress_logo();

                // Upload SplashScreen picture
                auto state = FileProcessor::create(&__boot_loader_end, SimpleTIM(32, 0, 0, 0));
                while(state.process((bytes_ready/sizeof(uint32_t))) == Progress::InProgress);

                Display::enable();
            }

            void setup() {
                GPU_IO::GP1.write(GPU_IO::Command::GP1::Reset());
                internal::Screen::configurate();
                internal::Screen::exchange_buffer_and_display();
                
                GPU::internal::wait_ready_for_CMD();
                GPU::internal::quick_fill_fast(Color24::Black(), PositionU16(32, 0), SizeU16(Display::Width, Display::Height));
            }
        }
    }
}