#include "BootLoader/boot_loader.hpp"
#include <PSX/System/IOPorts/dMa_io.hpp>

#include <PSX/Timer/high_res_timer.hpp>
#include <stdio.h>

namespace JabyEngine {
    namespace boot {
        namespace Start {    
            static void enable_DMA() {
                DMA_IO::DPCR.write(DMA_IO::DMAControlRegister{DMA_IO::DPCR.read() | DMA_IO::DMAControlRegister::SPUEnable | DMA_IO::DMAControlRegister::GPUEnable});
            }

            JabyEngine::NextRoutine setup() {
                enable_DMA();

                SPU::stop_voices();
                
                CD::setup();
                Timer::setup();

                const auto start = HighResTime::get_time_stamp();
                printf("Start...\n");
                GPU::setup();
                GPU::display_logo();
                const auto end   = HighResTime::get_time_stamp();
                printf("GPU setup took %ims %ius\n", start.milliseconds_to(end), start.microseconds_to(end));
                
                //Pause??

                SPU::setup();
                return BootFile::setup();
            }
        }
    }
}