#include <PSX/jabyengine_config.hpp>
#ifdef JABYENGINE_USE_HIGH_PERCISION_TIMER
    #include <PSX/System/IOPorts/interrupt_io.hpp>
    #include <PSX/System/syscalls.h>
    #define private public
    #include <PSX/Timer/high_res_timer.hpp>
    #undef private

    namespace JabyEngine {
        namespace Timer {
            extern InterrupCallback IRQCallback;
        }

        namespace boot {
            namespace Timer {
                using namespace JabyEngine::Timer;
                
                void setup() {
                    using namespace Timer_IO;

                    static constexpr auto Mode = CounterMode::with(CounterMode::FreeRun, Counter2::SyncMode::Freerun, CounterMode::ResetAfterTarget, CounterMode::IRQAtTarget, CounterMode::IRQEveryTime, CounterMode::IRQPulse, Counter2::Source::System_Clock_Div_8);

                    Interrupt::disable_irq(Interrupt::Timer2);

                    __syscall_EnterCriticalSection();
                        __syscall_SysEnqIntRP(Timer2Irq, &IRQCallback);
                    __syscall_ExitCriticalSection();

                    Counter2.target.write(CounterTarget::CounterTargetValue.with(HighResTime::TicksFor10ms));
                    Counter2.mode.write({Mode});

                    Interrupt::enable_irq(Interrupt::Timer2);
                }
            }
        }
    }
#else 
    namespace JabyEngine {
        namespace boot {
            namespace Timer {
                void setup() {
                }
            }
        }
    }
#endif //JABYENGINE_USE_HIGH_PERCISION_TIMER