#include "../../include/CD/cd_internal.hpp"
#include <PSX/System/IOPorts/interrupt_io.hpp>
#include <PSX/System/syscalls.h>

namespace JabyEngine {
    namespace CD {
        namespace internal {
            VolatilePOD<CD_IO::Interrupt::Type> last_interrupt{CD_IO::Interrupt::Type::None};

            static InterruptVerifierResult interrupt_verifier() {
                if(Interrupt::is_irq(Interrupt::CDROM)) {
                    const uint8_t old_idx = (CD_IO::IndexStatus.read() & 0x3);

                    CD_IO::PortIndex1::change_to();

                    last_interrupt.write(CD_IO::Interrupt::get_type(CD_IO::PortIndex1::InterruptFlagRegister));
                    CD_IO::Interrupt::ack(CD_IO::PortIndex1::InterruptFlagRegister);

                    CD_IO::IndexStatus.write({old_idx});
                    return InterruptVerifierResult::ExecuteHandler;
                }
                
                else {
                    return InterruptVerifierResult::SkipHandler;
                }
            }

            static void interrupt_handler(uint32_t) {
                Interrupt::ack_irq(Interrupt::CDROM);
                __syscall_ReturnFromException();
            }

            InterrupCallback callback = {
                .next              = nullptr,
                .handler_function  = reinterpret_cast<InterruptHandler>(interrupt_handler),
                .verifier_function = interrupt_verifier
            };
        }
    }
}