pub struct BitRange {
    pub start: usize,
    pub len:   usize,
}

impl BitRange {
    pub const fn from_to(start: usize, end: usize) -> BitRange {
        BitRange{start, len: (end - start + 1)}
    }
}

macro_rules! create_bit_functions {
    ($type_val:ty) => {
        paste::item! {
            const fn [< get_mask_ $type_val >](range: &BitRange) -> $type_val {
                (1 << range.len) - 1
            }

            pub const fn [< clear_value_ $type_val >](dst: $type_val, range: &BitRange) -> $type_val {
                dst & !([< get_mask_ $type_val >](range) << (range.start as $type_val))
            }

            pub const fn [< set_value_ $type_val >](dst: $type_val, value: $type_val, range: &BitRange) -> $type_val {
                [< clear_value_ $type_val >](dst, range) | ((value & [< get_mask_ $type_val >](range)) << range.start)
            }

            pub const fn [< get_value_ $type_val >](src: $type_val, range: &BitRange) -> $type_val {
                (src & [< get_mask_ $type_val >](range)) >> range.start
            }

            pub const fn [< bit_set_ $type_val >](src: $type_val, bit: usize) -> bool {
                src & (1 << bit) != 0
            }
        }
    };
}

create_bit_functions!(u8);
create_bit_functions!(u16);
create_bit_functions!(u32);